/*
 * © 2024 Browsolate Limited. All rights reserved.
 *
 * This source code is the property of Browsolate Limited and is strictly confidential.
 * Unauthorized copying, modification, distribution, or use of this code, in part or in whole,
 * without prior written permission from Browsolate Limited is strictly prohibited and may
 * result in legal action.
 *
 * For licensing inquiries, please contact Browsolate Limited at https://browsolate.com.
 *
 * Author: Browsolate Limited
 * Version: 1.0
 * Date: 2024-11-02
 */

import { updateRedirectionRule, clearAllRules, logCurrentRules } from './declarative-net.js';
import { getValidHttpUrl, _matchPattern, getStoredValue, extractFilenameFromUrl, generateEncryptedUrl, checkB8Host } from './common.js';


async function isExcludedDomain(url) {
    return _matchPattern(url, await getStoredValue("excludeDomains"));
}

async function isIncludedDomain(url) {
    return _matchPattern(url, await getStoredValue("includeDomains"));
}

async function processBeforeRequest(details){
    let b8Host = await getStoredValue("b8Host");
    if (!b8Host) {
        return;
    } else {
        b8Host = b8Host.replace(/\/+$/, '');
    }
    const extLogging = await getStoredValue("extLogging");

    let b8Url = new URL(b8Host);

    const userUrl = getValidHttpUrl(details.url);

    if(!userUrl){
        return;
    }

    if(userUrl.hostname === b8Url.hostname) {
        return;
    }

    if(await isExcludedDomain(userUrl)) {
        if(extLogging) {
            console.log(`Domain is Excluded: ${userUrl}`);
        }
        return;
    }

    if(await _maybeIsolateDownloadUrl(userUrl, 'Unknown', extLogging)) {
        _isolateUrl(userUrl, details.tabId, extLogging).then(()=>{
            if (extLogging) {
                console.log(`File download is isolated: ${userUrl}`);
            }
        }).catch(async (error) => {
            if(extLogging) {
                console.log(`Error generating encrypted URL for redirection: ${error.message} : ${userUrl}`);
            }
            await checkB8Host();
            await clearAllRules(extLogging);
        });
        return;
    }

    if(! await isIncludedDomain(userUrl)) {
        if(extLogging) {
            console.log(`Domain is not an included domain: ${userUrl}`);
        }
        return;
    }

    if(extLogging) {
        console.log(`Isolating URL: ${userUrl}`);
    }
    _isolateUrl(userUrl, details.tabId, extLogging).catch(async (error) => {
        if(extLogging) {
            console.log(`Error generating encrypted URL for redirection: ${error.message} : ${userUrl}`);
        }
        await checkB8Host();
        await clearAllRules(extLogging);
    });
}

function _isolateUrl(userUrl, tabId, extLogging) {
    // Generate the encrypted redirect URL based on the target URL
    return generateEncryptedUrl(userUrl.href).then(async (encryptedUrl) => {
        // Update the redirection rule with the new encrypted URL
        if(extLogging) {
            await logCurrentRules();
        }

        await updateRedirectionRule(userUrl.href, encryptedUrl.href, extLogging);

        if(extLogging) {
            await logCurrentRules();
        }

        if (tabId !== -1) {
            chrome.tabs.update(tabId, { url: encryptedUrl.href }, () => {
                if(extLogging) {
                    console.log(`Tab ${tabId} navigated to ${encryptedUrl}`);
                }
            });
        } else {
            chrome.tabs.create({ url: encryptedUrl.href, active: true });
        }
        //console.log(`Generated encrypted URL for redirection: ${encryptedUrl.href} tabId is ${tabId}`);
    });
}




async function _clearAllRules() {
    const extLogging = await getStoredValue("extLogging");
    return clearAllRules(extLogging);
}

chrome.webRequest.onBeforeRequest.addListener(processBeforeRequest, { urls: ["<all_urls>"], types: ["main_frame"]});
chrome.runtime.onStartup.addListener(async ()=>{
    if(!await checkPermissions()){
        await requestHostPermissions();
    }
    await _clearAllRules();
    chrome.alarms.create("checkB8Host", { periodInMinutes: 0.5 });
    await checkB8Host();
});
chrome.runtime.onInstalled.addListener(async () => {
    if(!await checkPermissions()){
        await requestHostPermissions();
    }
    await _clearAllRules();
}); // Also clear on installation or reload
chrome.storage.onChanged.addListener(async (changes, areaName) => {
    if (areaName === "sync") {
        await _clearAllRules();
        await checkB8Host();
    }
});

chrome.alarms.onAlarm.addListener(async (alarm) => {
    if (alarm.name === "checkB8Host") {
        await checkB8Host();
    }
});




//Download interception
async function _maybeIsolateDownloadUrl(url, mimeType, extLogging) {
    const filename = extractFilenameFromUrl(url);
    if(!filename) {
        return false; //Not a file
    }
    if(extLogging) {
        console.log(`Download detected: ${url} - ${filename} - MIME: ${mimeType}`);
    }

    let blockedMimeTypes = await getStoredValue("contentTypes");
    let extensionsToMatch = await getStoredValue("contentExtensions");
    return extensionsToMatch.some(ext => filename.toLowerCase().endsWith(ext)) ||
        blockedMimeTypes.includes(mimeType);
}

if(chrome.downloads && chrome.downloads.onDeterminingFilename){
    async function processDownload(downloadItem, suggest){
        const mimeType = downloadItem.mime || "Unknown";
        const extLogging = await getStoredValue("extLogging");
        if(await _maybeIsolateDownloadUrl(downloadItem.url, mimeType, extLogging)) {
            console.log(`Isolated download of ${downloadItem.url} with MIME type ${mimeType}`);
            _isolateUrl(new URL(downloadItem.url), -1, extLogging).then(() =>{
                chrome.downloads.cancel(downloadItem.id, () => {
                    if (chrome.runtime.lastError) {
                        console.error(`Error canceling download: ${chrome.runtime.lastError.message}`);
                    }
                });
            });
        } else { //Not Isolated
            suggest();
        }
    }
    chrome.downloads.onDeterminingFilename.addListener(processDownload);
}

chrome.runtime.onInstalled.addListener(() => {
    chrome.contextMenus.create({
        id: "open-in-browsolate",
        title: "Open in Browsolate",
        contexts: ["link"] // Show this menu only when right-clicking on links
    });
});

// Handle the click on the context menu
chrome.contextMenus.onClicked.addListener((info, tab) => {
    if (info.menuItemId === "open-in-browsolate") {
        const linkUrl = info.linkUrl; // The URL of the clicked link

        return getStoredValue("extLogging").then((extLogging) => {
            // Open the link in Browsolate
            return _isolateUrl(new URL(linkUrl), -1, extLogging);
        });
    }
});

async function requestHostPermissions() {
    // Check if the `browser.permissions` API is available (Firefox)
    if (typeof browser !== "undefined" && browser.permissions && browser.permissions.request) {
        try {
            const granted = await browser.permissions.request({
                permissions: [],
                origins: ["<all_urls>"]
            });

            if (granted) {
                console.log("Host permissions granted!");
            } else {
                console.log("Host permissions denied!");
            }
        } catch (error) {
            console.error("Error requesting permissions:", error);
        }
    } else {
        // Fallback for Chrome or other browsers
        console.log("Browser does not support `browser.permissions`. Assuming permissions are granted.");
    }
}


async function checkPermissions() {
    if (typeof browser !== "undefined" && browser.permissions) {
        try {
            const result = await browser.permissions.contains({
                origins: ["<all_urls>"]
            });

            if (result) {
                return true;
            }
        } catch (error) {
            console.error("Error checking permissions:", error);
        }
    } else {
        return true;
    }
    return false;
}