/*
 * © 2024 Browsolate Limited. All rights reserved.
 *
 * This source code is the property of Browsolate Limited and is strictly confidential.
 * Unauthorized copying, modification, distribution, or use of this code, in part or in whole,
 * without prior written permission from Browsolate Limited is strictly prohibited and may
 * result in legal action.
 *
 * For licensing inquiries, please contact Browsolate Limited at https://browsolate.com.
 *
 * Author: Browsolate Limited
 * Version: 1.0
 * Date: 2024-11-02
 */

// Generate a random IV for AES encryption
function generateIV() {
    const iv = new Uint8Array(16); // 16 bytes for AES-256-CBC
    self.crypto.getRandomValues(iv);
    return iv;
}

export async function encrypt(base64Key, plaintext) {
    // Convert the Base64 key to Uint8Array
    const key = base64ToUint8Array(base64Key);

    const iv = generateIV();
    const encoder = new TextEncoder();
    const data = encoder.encode(plaintext);

    // Import the key for AES encryption
    const cryptoKey = await self.crypto.subtle.importKey(
        "raw",
        key,
        { name: "AES-CBC" },
        false,
        ["encrypt"]
    );

    // Encrypt the data with AES-CBC
    const encrypted = await self.crypto.subtle.encrypt(
        { name: "AES-CBC", iv: iv },
        cryptoKey,
        data
    );

    // Concatenate IV + encrypted data into a single Uint8Array
    const ivAndEncryptedData = new Uint8Array([...iv, ...new Uint8Array(encrypted)]);

    // Return URL-safe Base64 encoded result
    return base64url(ivAndEncryptedData);
}

// Helper function to convert a Base64 string to a Uint8Array
function base64ToUint8Array(base64String) {
    const binaryString = atob(base64String);
    const bytes = new Uint8Array(binaryString.length);
    for (let i = 0; i < binaryString.length; i++) {
        bytes[i] = binaryString.charCodeAt(i);
    }
    return bytes;
}

// Helper function to convert to URL-safe Base64
function base64url(buffer) {
    // Convert the buffer to a standard Base64 string
    const base64String = btoa(
        String.fromCharCode(...new Uint8Array(buffer))
    );

    // Convert to URL-safe Base64
    return base64String.replace(/\+/g, "-").replace(/\//g, "_").replace(/=+$/, "");
}

// Call the API for encryption
export async function apiEncrypt(baseUrl, keyId, jsonData, password, ssl_verify = true) {
    const apiUrl = `${baseUrl}:5443/encrypt`;
    const auth = 'Basic ' + btoa(`b8admin:${password}`);

    try {
        const response = await fetch(apiUrl, {
            method: 'POST',
            headers: {
                'Authorization': auth,
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                keyName: keyId,
                data: jsonData
            })
        });

        if (!response.ok) {
            throw new Error(`Failed to encrypt data via API: ${response.statusText}`);
        }

        const responseData = await response.json();

        if (!responseData.encryptedData) {
            throw new Error(`Unexpected response format: ${JSON.stringify(responseData)}`);
        }

        return responseData.encryptedData;
    } catch (error) {
        throw error;
    }
}
