/*
 * © 2024 Browsolate Limited. All rights reserved.
 *
 * This source code is the property of Browsolate Limited and is strictly confidential.
 * Unauthorized copying, modification, distribution, or use of this code, in part or in whole,
 * without prior written permission from Browsolate Limited is strictly prohibited and may
 * result in legal action.
 *
 * For licensing inquiries, please contact Browsolate Limited at https://browsolate.com.
 *
 * Author: Browsolate Limited
 * Version: 1.0
 * Date: 2024-11-02
 */

import { loadViewerOptions } from './common.js';

document.addEventListener("DOMContentLoaded", async () => {
    setupTabs();
    await resetForm();
    document.getElementById("b8Host").addEventListener("input", handleHostChange);
    document.getElementById("config-form").addEventListener("submit", saveConfig);
    document.getElementById("cancelButton").addEventListener("click", cancelOptions);
});


async function resetForm(){
    const coreConfig = await loadCoreConfig();
    setCoreConfigFields(coreConfig);

    if (coreConfig.b8Host) {
        const viewerOptions = await loadViewerOptions(coreConfig.b8Host);
        let mergedOptions = {};
        if(viewerOptions && viewerOptions.length > 0) {
            mergedOptions = mergeStoredValuesWithDefaults(viewerOptions, coreConfig.viewerOptions || []);
        }
        createViewerOptionsFields(mergedOptions);
    }
}

// Cancel options and reset the form
async function cancelOptions(event) {
    event.preventDefault();
    await resetForm();
    chrome.storage.sync.get(null, (items) => { // `null` retrieves all keys and values
        chrome.storage.sync.set(items, () => {
            if (chrome.runtime.lastError) {
                console.error(`Error saving storage: ${chrome.runtime.lastError.message}`);
            } else {
                console.log("All settings have been successfully re-saved.");
            }
        });
    });
    window.close(); // Comment this line if you want to keep the dialog open
}

async function handleHostChange() {
    const b8Host = document.getElementById("b8Host").value.trim();
    clearErrorMessage(); // Clear any previous errors
    if (b8Host) {
        try {
            const viewerOptions = await loadViewerOptions(b8Host);
            const coreConfig = await loadCoreConfig();
            const mergedOptions = mergeStoredValuesWithDefaults(viewerOptions, coreConfig.viewerOptions || []);
            createViewerOptionsFields(mergedOptions);
            enableSaveButton(); // Enable the Save button
        } catch (error) {
            //console.error("Error loading viewer options:", error);
            displayErrorMessage("Failed to load viewer options. Please check the host URL.");
            createViewerOptionsFields({});
            disableSaveButton();
        }
    }
}
// Fetch core configuration from chrome.storage
async function loadCoreConfig() {
    return new Promise((resolve) => {
        chrome.storage.sync.get(
            ["b8Host", "b8MgmtPassword", "b8KeyName", "b8KeyValue",
                "failureMode", "includeDomains", "excludeDomains",
                    "contentTypes", "contentExtensions", "viewerOptions", "extLogging"],
            (items) => {
                resolve(items);
            }
        );
    });
}

// Merge stored values with defaults from viewer_options.json
function mergeStoredValuesWithDefaults(defaultOptions, storedOptions) {
    return defaultOptions.map(defaultOption => {
        const storedOption = storedOptions.find(opt => opt.name === defaultOption.name);
        return {
            ...defaultOption,
            value: storedOption ? storedOption.value : defaultOption.defaultValue
        };
    });
}

// Set core configuration fields from chrome.storage
function setCoreConfigFields(config) {
    document.getElementById("b8Host").value = config.b8Host || "";
    document.getElementById("b8MgmtPassword").value = config.b8MgmtPassword || "";
    document.getElementById("b8KeyName").value = config.b8KeyName || "default";
    document.getElementById("b8KeyValue").value = config.b8KeyValue || "";
    document.getElementById("extLogging").checked = config.extLogging;
    document.getElementById("failureMode").value = config.failureMode || "Open";
    document.getElementById("includeDomains").value = (config.includeDomains || []).join(",\n");
    document.getElementById("excludeDomains").value = (config.excludeDomains || []).join(",\n");
    document.getElementById("contentExtensions").value = (config.contentExtensions || []).join(",\n");
    document.getElementById("contentTypes").value = (config.contentTypes || []).join(",\n");
}

// Dynamically create form fields for viewerOptions with merged values
function createViewerOptionsFields(viewerOptions) {
    const container = document.getElementById("viewer-options-container");
    const advanced_container = document.getElementById("viewer-options-advanced-container");
    const checkbox_container = document.getElementById("viewer-options-checkbox-container");
    const advanced_checkbox_container = document.getElementById("viewer-options-advanced-checkbox-container");

    container.innerHTML = "";
    advanced_container.innerHTML = "";
    checkbox_container.innerHTML = "";
    advanced_checkbox_container.innerHTML = "";


    if(Object.keys(viewerOptions).length === 0) {
        checkbox_container.innerHTML = "Please enter a valid Browsolate host (root) URL in System Settings.";
        return;
    }

    viewerOptions.forEach(param => {
        const paramGroup = document.createElement("div");
        paramGroup.classList.add("parameter-group");

        const ignoredFields = ["url", "customRequestHeaders"];

        if (!ignoredFields.includes(param.name)) {
            const label = document.createElement("label");
            label.textContent = `${param.name}: `;
            paramGroup.appendChild(label);

            const value = param.value !== undefined ? param.value : param.defaultValue;

            if (param.type === "boolean") {
                const checkbox = document.createElement("input");
                checkbox.type = "checkbox";
                checkbox.id = param.name;
                checkbox.checked = value === true || value === "true"; // Convert to boolean
                paramGroup.appendChild(checkbox);
            } else if (param.type === "integer") {
                const input = document.createElement("input");
                input.type = "number";
                input.id = param.name;
                input.value = parseInt(value, 10) || param.defaultValue; // Convert to integer
                paramGroup.appendChild(input);
            } /*else if (param.type === "string" && param.name === "themeName") {
                const select = document.createElement("select");
                select.id = param.name;

                // Define available theme options (this could be modified or populated dynamically)
                const themeOptions = [
                    { name: "default", displayName: "Default Theme" },
                    { name: "dark", displayName: "Dark Theme" },
                    { name: "light", displayName: "Light Theme" }
                ];

                themeOptions.forEach(theme => {
                    const option = document.createElement("option");
                    option.value = theme.name;
                    option.textContent = theme.displayName;
                    if (theme.name === value) {
                        option.selected = true;
                    }
                    select.appendChild(option);
                });
                paramGroup.appendChild(select);
            } */ else if (param.type === "abstime") {
                const select = document.createElement("select");
                select.id = param.name;

                // Define time options
                const timeOptions = [
                    {text: "Infinite", value: -1},
                    {text: "1 minute", value: 1 * 60},
                    {text: "5 minutes", value: 5 * 60},
                    {text: "15 minutes", value: 15 * 60},
                    {text: "1 hour", value: 1 * 60 * 60},
                    {text: "4 hours", value: 4 * 60 * 60},
                    {text: "12 hours", value: 12 * 60 * 60},
                    {text: "24 hours", value: 24 * 60 * 60},
                    {text: "1 week", value: 7 * 24 * 60 * 60},
                    {text: "1 month", value: 30 * 24 * 60 * 60},
                    {text: "1 year", value: 365 * 24 * 60 * 60}
                ];

                timeOptions.forEach(optionData => {
                    const option = document.createElement("option");
                    option.value = optionData.value;
                    option.textContent = optionData.text;
                    if (parseInt(value, 10) === optionData.value) {
                        option.selected = true;
                    }
                    select.appendChild(option);
                });
                paramGroup.appendChild(select);
            } else {
                const input = document.createElement("input");
                input.type = "text";
                input.id = param.name;
                input.value = value;
                paramGroup.appendChild(input);
            }
        }
        if(param.type === "boolean") {
            if(param.advanced) {
                advanced_checkbox_container.appendChild(paramGroup);
            } else {
                checkbox_container.appendChild(paramGroup);
            }
        } else {
            if(param.advanced) {
                advanced_container.appendChild(paramGroup);
            } else {
                container.appendChild(paramGroup);
            }
        }
    });
}

// Show error message in the UI
function displayErrorMessage(message) {
    const errorMessageElement = document.getElementById("error-message");
    errorMessageElement.textContent = message;
    errorMessageElement.style.display = "block";
}

// Clear any existing error message
function clearErrorMessage() {
    const errorMessageElement = document.getElementById("error-message");
    errorMessageElement.textContent = "";
    errorMessageElement.style.display = "none";
}

// Enable the Save button
function enableSaveButton() {
    document.getElementById("saveButton").disabled = false;
}

function disableSaveButton() {
    document.getElementById("saveButton").disabled = true;
}

function _textareaToArray(elementName) {
    const patternsText = document.getElementById(elementName).value;
    return patternsText
        .split(/[\n,]+/) // Split by commas or new lines
        .map(pattern => pattern.trim())
        .filter(pattern => pattern.length > 0); // Remove empty patterns
}

// Save configuration to chrome.storage
// Save configuration to chrome.storage and close the options dialog
async function saveConfig(event) {
    event.preventDefault();


    // Fetch viewer_options.json to get the types of each parameter
    const viewerOptionsJson = await loadViewerOptions(document.getElementById("b8Host").value);

    // Collect viewerOptions values with type handling
    const viewerOptions = [];
    if(viewerOptionsJson && viewerOptionsJson.length > 0) {
        document.querySelectorAll("#viewer-options-container .parameter-group, #viewer-options-checkbox-container .parameter-group").forEach(group => {
            const input = group.querySelector("input, select");
            if (input) {
                const paramName = input.id;

                // Find the type in viewerOptionsJson
                const optionDefinition = viewerOptionsJson.find(option => option.name === paramName);
                const type = optionDefinition ? optionDefinition.type : "string"; // Default to string if not found

                let value;
                if (input.type === "checkbox") {
                    value = input.checked; // Boolean
                } else if (input.type === "number") {
                    value = parseInt(input.value, 10); // Integer
                } else if (input.type === "select-one") {
                    value = input.value === "-1" ? -1 : parseInt(input.value, 10) || input.value; // Handle select values for abstime or themeName
                } else {
                    value = input.value; // String
                }

                viewerOptions.push({ name: paramName, value: value, type: type });
            }
        });
    }

    const config = {
        b8Host: document.getElementById("b8Host").value,
        b8MgmtPassword: document.getElementById("b8MgmtPassword").value,
        b8KeyName: document.getElementById("b8KeyName").value,
        b8KeyValue: document.getElementById("b8KeyValue").value,
        failureMode: document.getElementById("failureMode").value,
        includeDomains:  _textareaToArray("includeDomains"),
        excludeDomains: _textareaToArray("excludeDomains"),
        contentTypes: _textareaToArray("contentTypes"),
        contentExtensions: _textareaToArray("contentExtensions"),
        viewerOptions: viewerOptions,
        extLogging: document.getElementById("extLogging").checked
    };

    chrome.storage.sync.set(config, () => {
        console.log("Settings saved.");
        window.close(); // Close the options dialog after saving
    });
}
function setupTabs() {
    const tabs = document.querySelectorAll('.tab');
    const tabContents = document.querySelectorAll('.tab-content');

    tabs.forEach(tab => {
        tab.addEventListener('click', () => {
            // Deactivate all tabs
            tabs.forEach(t => t.classList.remove('active'));
            // Deactivate all tab contents
            tabContents.forEach(content => content.classList.remove('active'));

            // Activate clicked tab
            tab.classList.add('active');
            // Activate corresponding tab content
            const target = tab.getAttribute('data-tab');
            document.getElementById(target).classList.add('active');
        });
    });
}